;*****************************************************************************
;I2C Routines for 8051 v1.1
;
;routines for simple access to the I2C-Bus 
;
;usage:
;	write Read/Write adress for Slave in ACC 
;	call I2C_Start 
;	call I2C_Write
;
;	Read:	
;	to read a byte call I2C_Read; to write a byte the last Byte call I2C_Read_Last (I2C_Read_Last ;	sends a NACK and not a ACK to the slave --> I2C-Protocol
;	(Data are saved in ACC) 
;	call I2C_Stop	
;
;	
;	Write:
;	Data in ACC	
;	call I2C_Write			
;	call I2C_Stop	
;
;The comments are still in german but I'm too lazy to translate it; but the routines are rather simple
;(c) Clemens Hofbauer 5.AHE 1999  <c_hofi@gmx.net>
;*****************************************************************************
$NOMOD51
$INCLUDE(reg51.pdf)

PUBLIC I2C_Start,I2C_Stop,I2C_Read,I2C_Read_Last,I2C_Write

I2C_Code SEGMENT CODE
SBitCnt SEGMENT DATA

SDA BIT P1.0
SCL BIT P1.1
RSEG SBitCnt

BitCnt: DS 1


RSEG I2C_Code
;*****************************************************************************
;Startbedingung am Bus erzeugen
;modifizierte Register: keine
;*****************************************************************************
I2C_Start:
	setb SCL	;Bus inaktiv
	setb SDA	;Bus inaktiv
	call I2C_Wait
	clr SDA		;Start-Signal erzeugen 
	call I2C_Wait
	clr SCL		;SDA darf sich ndern
	ret
;*****************************************************************************
;Stopbedingung am Bus erzeugen
;modifizierte Rgister: keine
;*****************************************************************************
I2C_Stop:
	clr SDA
	setb SCL	;Vorbereiten fr Stopbedingung
	call I2C_Wait
	setb SDA	;Stop-Signal erzeugen
	ret

;*****************************************************************************
;Routine fr Schreibzugriff auf I2C-Bus
;modifizierte Rgister: ACC,R1,BitCnt,Carry 
;*****************************************************************************
I2C_Write:
	mov R1,A	;A sichern
	mov BitCnt,#8	;Schleifenzhler laden
I2C_Write_Loop:
	rlc A		;Bit in Carry schieben
	mov SDA,C	;Carry auf Bus legen
	nop		;Warten
	setb SCL	;Takt erzeugen
	call I2C_Wait
I2C_Write_Loop_Wait:
	jnb SCL,$	;Takverlngerung von Slave beachten
	clr SCL		;SDA darf sich ndern
	djnz BitCnt,I2C_Write_Loop	;Ende der Schleife
	nop
	call I2C_Ack_Read	;ACK einlesen
	jnc label		;Ende bei erfolgreichem Schreiben
	mov A,R1		;sonst Schreiben wiederholen
	call I2C_Stop
	call I2C_Start
	jmp I2C_Write
label:	
	ret

;****************************************************************************
;Routine fr den Lesezugriff auf dem I2C-Bus
;modifizierte Register: ACC,BitCnt,Carry
;**************************************************************************** 
I2C_Read_Dummy:			;Read_Dummy wird durch I2C_Read aufgerufen
	setb SDA		;SDA freigeben
	clr A			;ACC fr rlc lschen
	mov BitCnt,#8		;Zhler laden
I2C_Read_Loop:
	clr SCL
	call I2C_Wait
	setb SCL		;Takt erzeugen
I2C_Read_Check:
	jnb SCL,$		;Auf Takverlngerung des Slaves warten
	mov C,SDA		;SDA in Carry einlesen
	rlc A			;Bit aus Carry in ACC rotieren
	djnz BitCnt,I2C_Read_Loop	;Ende der Schleife
	clr SCL			;Daten drfen sich ndern
	ret

;*****************************************************************************
;Schreiben eines ACK-Signals des Masters 
;modifizierte Rgister: keine
;*****************************************************************************
I2C_Ack_Write:
	clr SDA		;SDA fr ACK auf LOW
	nop
	setb SCL	;Clock erzeugen
	call I2C_Wait	
	clr SCL		;Daten drfen sich ndern
	setb SDA	;SDA freigeben
	call I2C_Wait
	ret
;*****************************************************************************
;Lesen eines ACK-Signals vom Bus
;modifizierte Rgister: Carry
;*****************************************************************************
I2C_Ack_Read:
	setb SDA	;SDA freigeben
	nop
	setb SCL	;SCL Impuls
	call I2C_Wait
	mov C,SDA	;Status der SDA Leitung einlesen
	clr SCL
	nop
	ret

;*****************************************************************************
;NACK auf den Bus schreiben
;modifizierte Rgister: keine
;*****************************************************************************
I2C_Nack_Write:
	setb SDA	;SDA auf HIGH	
	nop
	setb SCL
	call I2C_Wait
	clr SCL		;Flanke auf SCL erzeugen
	call I2C_Wait
	ret 

;*****************************************************************************
;Leseroutine + ACK (fr einfacheren Aufruf im Hauptprogramm)
;modifizierte Rgister: keine
;*****************************************************************************
I2C_Read:
	call I2C_Read_Dummy
	call I2C_Ack_Write
	ret

;*****************************************************************************
;Letzter Lesezugriff des Masters auf den Slave
;modifizierte Rgister: keine
;*****************************************************************************
I2C_Read_Last:
	call I2C_Read_Dummy
	call I2C_Nack_Write
	ret

;*****************************************************************************
;Warteschleife fr I2C Spezifikation
;modifizierte Register: keine
;*****************************************************************************
I2C_Wait:
	nop
	ret
END